@@ -1,6 +1,7 @@
 // Components for manipulating sequences of characters -*- C++ -*-
 
-// Copyright (C) 1997, 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
+// Copyright (C) 1997, 1998, 1999, 2000, 2001, 2002
+// Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -118,7 +119,6 @@ namespace std
       //   4. All fields==0 is an empty string, given the extra storage
       //      beyond-the-end for a null terminator; thus, the shared
       //      empty string representation needs no constructor.
-
       struct _Rep
       {
 	// Types:
@@ -531,29 +531,53 @@ namespace std
 
       basic_string& 
       insert(size_type __pos1, const basic_string& __str)
-      { 
-	iterator __p = _M_check(__pos1);
-	this->replace(__p, __p, __str._M_ibegin(), __str._M_iend());
-        return *this; 
-      }
+      { return this->insert(__pos1, __str, 0, __str.size()); }
 
       basic_string& 
       insert(size_type __pos1, const basic_string& __str,
 	     size_type __pos2, size_type __n)
-      { 
-	iterator __p = _M_check(__pos1);
-	this->replace(__p, __p, __str._M_check(__pos2), 
-		      __str._M_fold(__pos2, __n));
-        return *this; 
+      {
+	const size_type __strsize = __str.size();
+ 	if (__pos2 > __strsize)
+	  __throw_out_of_range("basic_string::insert");
+	const bool __testn = __n < __strsize - __pos2;
+	const size_type __newsize = __testn ? __n : __strsize - __pos2;
+	return this->insert(__pos1, __str._M_data() + __pos2, __newsize); 
       }
 
       basic_string& 
       insert(size_type __pos, const _CharT* __s, size_type __n)
-      { 
-	iterator __p = _M_check(__pos);
-	this->replace(__p, __p, __s, __s + __n);
-        return *this; 
-      }
+      {
+	const size_type __size = this->size();
+ 	if (__pos > __size)
+	  __throw_out_of_range("basic_string::insert");
+	if (__size > this->max_size() - __n)
+	  __throw_length_error("basic_string::insert");
+	if (_M_rep()->_M_is_shared() || less<const _CharT*>()(__s, _M_data())
+	    || less<const _CharT*>()(_M_data() + __size, __s))
+	  return _M_replace_safe(_M_ibegin() + __pos, _M_ibegin() + __pos,
+				 __s, __s + __n);
+	else
+	  {
+	    // Work in-place. If _M_mutate reallocates the string, __s
+	    // does not point anymore to valid data, therefore we save its
+	    // offset, then we restore it.
+	    const size_type __off = __s - _M_data();
+	    _M_mutate(__pos, 0, __n);
+	    __s = _M_data() + __off;
+	    _CharT* __p = _M_data() + __pos;
+	    if (__s  + __n <= __p)
+	      traits_type::copy(__p, __s, __n);
+	    else if (__s >= __p)
+	      traits_type::copy(__p, __s + __n, __n);
+	    else
+	      {
+		traits_type::copy(__p, __s, __p - __s);
+		traits_type::copy(__p + (__p - __s), __p + __n, __n - (__p - __s));
+	      }
+	    return *this;
+	  }
+       }
 
       basic_string&  
       insert(size_type __pos, const _CharT* __s)
@@ -602,10 +626,7 @@ namespace std
 
       basic_string& 
       replace(size_type __pos, size_type __n, const basic_string& __str)
-      { 
-	return this->replace(_M_check(__pos), _M_fold(__pos, __n),
-			      __str.begin(), __str.end()); 
-      }
+      { return this->replace(__pos, __n, __str._M_data(), __str.size()); }
 
       basic_string& 
       replace(size_type __pos1, size_type __n1, const basic_string& __str,
@@ -615,36 +636,41 @@ namespace std
       replace(size_type __pos, size_type __n1, const _CharT* __s,
 	      size_type __n2)
       { 
-	return this->replace(_M_check(__pos), _M_fold(__pos, __n1),
-			     __s, __s + __n2); 
+	const size_type __size = this->size();
+ 	if (__pos > __size)
+	  __throw_out_of_range("basic_string::replace");
+	if (__size - __n1 > this->max_size() - __n2)
+	  __throw_length_error("basic_string::replace");
+	const bool __testn1 = __n1 < __size - __pos;
+	const size_type __foldn1 = __testn1 ? __n1 : __size - __pos;
+	if (_M_rep()->_M_is_shared() || less<const _CharT*>()(__s, _M_data())
+	    || less<const _CharT*>()(_M_data() + __size, __s))
+	  return _M_replace_safe(_M_ibegin() + __pos,
+				 _M_ibegin() + __pos + __foldn1, __s, __s + __n2);	
+	else return this->replace(_M_check(__pos), _M_fold(__pos, __n1),
+				  __s, __s + __n2); 
       }
 
       basic_string& 
       replace(size_type __pos, size_type __n1, const _CharT* __s)
-      { 
-	return this->replace(_M_check(__pos), _M_fold(__pos, __n1),
-			     __s, __s + traits_type::length(__s)); 
-      }
+      { return this->replace(__pos, __n1, __s, traits_type::length(__s)); }
 
       basic_string& 
       replace(size_type __pos, size_type __n1, size_type __n2, _CharT __c)
-      { 
-	return this->replace(_M_check(__pos), _M_fold(__pos, __n1), __n2, __c);
-      }
+      { return this->replace(_M_check(__pos), _M_fold(__pos, __n1), __n2, __c); }
 
       basic_string& 
       replace(iterator __i1, iterator __i2, const basic_string& __str)
-      { return this->replace(__i1, __i2, __str.begin(), __str.end()); }
+      { return this->replace(__i1, __i2, __str._M_data(), __str.size()); }
 
       basic_string& 
       replace(iterator __i1, iterator __i2,
                            const _CharT* __s, size_type __n)
-      { return this->replace(__i1, __i2, __s, __s + __n); }
+      { return this->replace(__i1 - _M_ibegin(), __i2 - __i1, __s, __n); }
 
       basic_string& 
       replace(iterator __i1, iterator __i2, const _CharT* __s)
-      { return this->replace(__i1, __i2, __s, 
-			     __s + traits_type::length(__s)); }
+      { return this->replace(__i1, __i2, __s, traits_type::length(__s)); }
 
       basic_string& 
       replace(iterator __i1, iterator __i2, size_type __n, _CharT __c);
