@@ -1,6 +1,6 @@
 // Allocators -*- C++ -*-
 
-// Copyright (C) 2001 Free Software Foundation, Inc.
+// Copyright (C) 2001, 2002 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -49,8 +49,8 @@
 #define __GLIBCPP_INTERNAL_ALLOC_H
 
 /**
+ *  @defgroup Allocators Memory Allocators
  *  @maint
- *  @addtogroup Allocators
  *  stl_alloc.h implements some node allocators.  These are NOT the same as
  *  allocators in the C++ standard, nor in the original H-P STL.  They do not
  *  encapsulate different pointer types; we assume that there is only one
@@ -72,16 +72,17 @@
  *
  *  "SGI" allocators may be wrapped in __allocator to convert the interface
  *  into a "standard" one.
- *
  *  @endmaint
+ *
+ *  The canonical description of these classes is in docs/html/ext/howto.html
+ *  or online at http://gcc.gnu.org/onlinedocs/libstdc++/ext/howto.html#3
 */
 
-
-#include <bits/functexcept.h>   // for __throw_bad_alloc
-#include <bits/std_cstddef.h>
-#include <bits/std_cstdlib.h>
-#include <bits/std_cstring.h>
-#include <bits/std_cassert.h>
+#include <cstddef>
+#include <cstdlib>
+#include <cstring>
+#include <cassert>
+#include <bits/functexcept.h>   // For __throw_bad_alloc
 #include <bits/stl_threads.h>
 
 namespace std
@@ -91,8 +92,8 @@ namespace std
    *  A new-based allocator, as required by the standard.  Allocation and
    *  deallocation forward to global new and delete.  "SGI" style, minus
    *  reallocate().
-   *  (See @link Allocators allocators info @endlink for more.)
    *  @endmaint
+   *  (See @link Allocators allocators info @endlink for more.)
   */
   class __new_alloc 
   {
@@ -114,8 +115,8 @@ namespace std
    *  storage efficient.  The template argument is unused and is only present
    *  to permit multiple instantiations (but see __default_alloc_template
    *  for caveats).  "SGI" style, plus __set_malloc_handler for OOM conditions.
-   *  (See @link Allocators allocators info @endlink for more.)
    *  @endmaint
+   *  (See @link Allocators allocators info @endlink for more.)
   */
   template <int __inst>
     class __malloc_alloc_template 
@@ -212,8 +213,8 @@ namespace std
    *
    *  This is neither "standard"-conforming nor "SGI".  The _Alloc parameter
    *  must be "SGI" style.
-   *  (See @link Allocators allocators info @endlink for more.)
    *  @endmaint
+   *  (See @link Allocators allocators info @endlink for more.)
   */
   template<class _Tp, class _Alloc>
   class __simple_alloc
@@ -244,8 +245,8 @@ namespace std
    *  "There is some evidence that this can confuse Purify." - SGI comment
    *
    *  This adaptor is "SGI" style.  The _Alloc parameter must also be "SGI".
-   *  (See @link Allocators allocators info @endlink for more.)
    *  @endmaint
+   *  (See @link Allocators allocators info @endlink for more.)
   */
   template <class _Alloc>
   class __debug_alloc
@@ -317,270 +318,277 @@ typedef __mem_interface __single_client_alloc;
  *  approach.  If you do not wish to share the free lists with the main
  *  default_alloc instance, instantiate this with a non-zero __inst.
  *
- *  (See @link Allocators allocators info @endlink for more.)
  *  @endmaint
+ *  (See @link Allocators allocators info @endlink for more.)
 */
-template <bool __threads, int __inst>
-class __default_alloc_template
-{
+template<bool __threads, int __inst>
+  class __default_alloc_template
+  {
+  private:
+    enum {_ALIGN = 8};
+    enum {_MAX_BYTES = 128};
+    enum {_NFREELISTS = _MAX_BYTES / _ALIGN};
+    
+    union _Obj 
+    {
+      union _Obj* _M_free_list_link;
+      char        _M_client_data[1];    // The client sees this.
+    };
 
-private:
-  enum {_ALIGN = 8};
-  enum {_MAX_BYTES = 128};
-  enum {_NFREELISTS = _MAX_BYTES / _ALIGN};
+    static _Obj* volatile 	_S_free_list[_NFREELISTS]; 
 
-  static size_t
-  _S_round_up(size_t __bytes) 
+    // Chunk allocation state.
+    static char* 		_S_start_free;
+    static char* 		_S_end_free;
+    static size_t 		_S_heap_size;
+    
+    static _STL_mutex_lock 	_S_node_allocator_lock;
+
+    static size_t
+    _S_round_up(size_t __bytes) 
     { return (((__bytes) + (size_t) _ALIGN-1) & ~((size_t) _ALIGN - 1)); }
 
-  union _Obj {
-    union _Obj* _M_free_list_link;
-    char        _M_client_data[1];    // The client sees this.
-  };
-
-  static _Obj* volatile _S_free_list[]; 
-  static size_t _S_freelist_index(size_t __bytes)
+    static size_t 
+    _S_freelist_index(size_t __bytes)
     { return (((__bytes) + (size_t)_ALIGN-1)/(size_t)_ALIGN - 1); }
 
-  // Returns an object of size __n, and optionally adds to size __n free list.
-  static void* _S_refill(size_t __n);
-  // Allocates a chunk for nobjs of size size.  nobjs may be reduced
-  // if it is inconvenient to allocate the requested number.
-  static char* _S_chunk_alloc(size_t __size, int& __nobjs);
+    // Returns an object of size __n, and optionally adds to size __n
+    // free list.
+    static void* 
+    _S_refill(size_t __n);
 
-  // Chunk allocation state.
-  static char* _S_start_free;
-  static char* _S_end_free;
-  static size_t _S_heap_size;
-
-  static _STL_mutex_lock _S_node_allocator_lock;
-
-  // It would be nice to use _STL_auto_lock here.  But we need a test whether
-  // threads are in use.
-  class _Lock {
+    // Allocates a chunk for nobjs of size size.  nobjs may be reduced
+    // if it is inconvenient to allocate the requested number.
+    static char* 
+    _S_chunk_alloc(size_t __size, int& __nobjs);
+    
+    // It would be nice to use _STL_auto_lock here.  But we need a
+    // test whether threads are in use.
+    class _Lock 
+    {
     public:
       _Lock() { if (__threads) _S_node_allocator_lock._M_acquire_lock(); }
       ~_Lock() { if (__threads) _S_node_allocator_lock._M_release_lock(); }
-  } __attribute__ ((__unused__));
-  friend class _Lock;
-
-public:
-
-  // __n must be > 0
-  static void* allocate(size_t __n)
-  {
-    void* __ret = 0;
-
-    if (__n > (size_t) _MAX_BYTES) 
-      __ret = __mem_interface::allocate(__n);
-    else 
-      {
-	_Obj* volatile* __my_free_list = _S_free_list + _S_freelist_index(__n);
-	// Acquire the lock here with a constructor call.  This ensures that
-	// it is released in exit or during stack unwinding.
-	_Lock __lock_instance;
-	_Obj* __restrict__ __result = *__my_free_list;
-	if (__result == 0)
-	  __ret = _S_refill(_S_round_up(__n));
-	else 
-	  {
-	    *__my_free_list = __result -> _M_free_list_link;
-	    __ret = __result;
-	  }
-      }
+    } __attribute__ ((__unused__));
+    friend class _Lock;
     
-    return __ret;
+  public:
+    // __n must be > 0
+    static void* 
+    allocate(size_t __n)
+    {
+      void* __ret = 0;
+      
+      if (__n > (size_t) _MAX_BYTES) 
+	__ret = __mem_interface::allocate(__n);
+      else 
+	{
+	  _Obj* volatile* __my_free_list = _S_free_list 
+	    + _S_freelist_index(__n);
+	  // Acquire the lock here with a constructor call.  This
+	  // ensures that it is released in exit or during stack
+	  // unwinding.
+	  _Lock __lock_instance;
+	  _Obj* __restrict__ __result = *__my_free_list;
+	  if (__result == 0)
+	    __ret = _S_refill(_S_round_up(__n));
+	  else 
+	    {
+	      *__my_free_list = __result -> _M_free_list_link;
+	      __ret = __result;
+	    }
+	}
+      return __ret;
+    };
+
+    // __p may not be 0
+    static void 
+    deallocate(void* __p, size_t __n)
+    {
+      if (__n > (size_t) _MAX_BYTES)
+	__mem_interface::deallocate(__p, __n);
+      else 
+	{
+	  _Obj* volatile*  __my_free_list
+	    = _S_free_list + _S_freelist_index(__n);
+	  _Obj* __q = (_Obj*)__p;
+	  
+	  // Acquire the lock here with a constructor call.  This ensures that
+	  // it is released in exit or during stack unwinding.
+	  _Lock __lock_instance;
+	  __q -> _M_free_list_link = *__my_free_list;
+	  *__my_free_list = __q;
+	}
+    }
+    
+    static void* 
+    reallocate(void* __p, size_t __old_sz, size_t __new_sz);
   };
 
-  // __p may not be 0
-  static void deallocate(void* __p, size_t __n)
-  {
-    if (__n > (size_t) _MAX_BYTES)
-      __mem_interface::deallocate(__p, __n);
-    else 
-      {
-	_Obj* volatile*  __my_free_list
-          = _S_free_list + _S_freelist_index(__n);
-	_Obj* __q = (_Obj*)__p;
-	
-	// Acquire the lock here with a constructor call.  This ensures that
-	// it is released in exit or during stack unwinding.
-	_Lock __lock_instance;
-	__q -> _M_free_list_link = *__my_free_list;
-	*__my_free_list = __q;
-      }
-  }
-  
-  static void* reallocate(void* __p, size_t __old_sz, size_t __new_sz);
-};
+
+  template<bool __threads, int __inst>
+    inline bool 
+    operator==(const __default_alloc_template<__threads, __inst>&,
+	       const __default_alloc_template<__threads, __inst>&)
+    { return true; }
+
+  template<bool __threads, int __inst>
+    inline bool 
+    operator!=(const __default_alloc_template<__threads, __inst>&,
+	       const __default_alloc_template<__threads, __inst>&)
+    { return false; }
 
 
-template <bool __threads, int __inst>
-inline bool operator==(const __default_alloc_template<__threads, __inst>&,
-                       const __default_alloc_template<__threads, __inst>&)
-{
-  return true;
-}
-
-template <bool __threads, int __inst>
-inline bool operator!=(const __default_alloc_template<__threads, __inst>&,
-                       const __default_alloc_template<__threads, __inst>&)
-{
-  return false;
-}
-
-
-// We allocate memory in large chunks in order to avoid fragmenting the
-// malloc heap (or whatever __mem_interface is using) too much.  We assume
-// that __size is properly aligned.  We hold the allocation lock.
-template <bool __threads, int __inst>
-char*
-__default_alloc_template<__threads, __inst>::_S_chunk_alloc(size_t __size, 
-                                                            int& __nobjs)
-{
-    char* __result;
-    size_t __total_bytes = __size * __nobjs;
-    size_t __bytes_left = _S_end_free - _S_start_free;
-
-    if (__bytes_left >= __total_bytes) 
+  // We allocate memory in large chunks in order to avoid fragmenting the
+  // malloc heap (or whatever __mem_interface is using) too much.  We assume
+  // that __size is properly aligned.  We hold the allocation lock.
+  template<bool __threads, int __inst>
+    char*
+    __default_alloc_template<__threads, __inst>::_S_chunk_alloc(size_t __size, 
+								int& __nobjs)
+    {
+      char* __result;
+      size_t __total_bytes = __size * __nobjs;
+      size_t __bytes_left = _S_end_free - _S_start_free;
+      
+      if (__bytes_left >= __total_bytes) 
       {
         __result = _S_start_free;
         _S_start_free += __total_bytes;
         return(__result);
       } 
-    else if (__bytes_left >= __size) 
-      {
-        __nobjs = (int)(__bytes_left/__size);
-        __total_bytes = __size * __nobjs;
-        __result = _S_start_free;
-        _S_start_free += __total_bytes;
-        return(__result);
-    } 
-    else 
-      {
-        size_t __bytes_to_get = 
-	  2 * __total_bytes + _S_round_up(_S_heap_size >> 4);
-        // Try to make use of the left-over piece.
-        if (__bytes_left > 0) 
-	  {
-	    _Obj* volatile* __my_free_list =
-	      _S_free_list + _S_freelist_index(__bytes_left);
-	    
-	    ((_Obj*)_S_start_free) -> _M_free_list_link = *__my_free_list;
-	    *__my_free_list = (_Obj*)_S_start_free;
-	  }
-        _S_start_free = (char*) __mem_interface::allocate(__bytes_to_get);
-        if (0 == _S_start_free) 
-	  {
-            size_t __i;
-            _Obj* volatile* __my_free_list;
-	    _Obj* __p;
-            // Try to make do with what we have.  That can't hurt.  We
-            // do not try smaller requests, since that tends to result
-            // in disaster on multi-process machines.
-	    __i = __size;
-            for (; __i <= (size_t) _MAX_BYTES; __i += (size_t) _ALIGN) 
-	      {
-                __my_free_list = _S_free_list + _S_freelist_index(__i);
-                __p = *__my_free_list;
-                if (0 != __p) 
-		  {
-		    *__my_free_list = __p -> _M_free_list_link;
-		    _S_start_free = (char*)__p;
-                    _S_end_free = _S_start_free + __i;
-                    return(_S_chunk_alloc(__size, __nobjs));
-                    // Any leftover piece will eventually make it to the
-                    // right free list.
-		  }
-	      }
-	    _S_end_free = 0;	// In case of exception.
-            _S_start_free = (char*)__mem_interface::allocate(__bytes_to_get);
-            // This should either throw an exception or remedy the situation.
-            // Thus we assume it succeeded.
-	  }
-        _S_heap_size += __bytes_to_get;
-        _S_end_free = _S_start_free + __bytes_to_get;
-        return(_S_chunk_alloc(__size, __nobjs));
-      }
-}
-
-
-// Returns an object of size __n, and optionally adds to "size __n"'s free list.
-// We assume that __n is properly aligned.  We hold the allocation lock.
-template <bool __threads, int __inst>
-void*
-__default_alloc_template<__threads, __inst>::_S_refill(size_t __n)
-{
-    int __nobjs = 20;
-    char* __chunk = _S_chunk_alloc(__n, __nobjs);
-    _Obj* volatile* __my_free_list;
-    _Obj* __result;
-    _Obj* __current_obj;
-    _Obj* __next_obj;
-    int __i;
-
-    if (1 == __nobjs) return(__chunk);
-    __my_free_list = _S_free_list + _S_freelist_index(__n);
-
-    /* Build free list in chunk */
+      else if (__bytes_left >= __size) 
+	{
+	  __nobjs = (int)(__bytes_left/__size);
+	  __total_bytes = __size * __nobjs;
+	  __result = _S_start_free;
+	  _S_start_free += __total_bytes;
+	  return(__result);
+	} 
+      else 
+	{
+	  size_t __bytes_to_get = 
+	    2 * __total_bytes + _S_round_up(_S_heap_size >> 4);
+	  // Try to make use of the left-over piece.
+	  if (__bytes_left > 0) 
+	    {
+	      _Obj* volatile* __my_free_list =
+		_S_free_list + _S_freelist_index(__bytes_left);
+	      
+	      ((_Obj*)_S_start_free) -> _M_free_list_link = *__my_free_list;
+	      *__my_free_list = (_Obj*)_S_start_free;
+	    }
+	  _S_start_free = (char*) __mem_interface::allocate(__bytes_to_get);
+	  if (0 == _S_start_free) 
+	    {
+	      size_t __i;
+	      _Obj* volatile* __my_free_list;
+	      _Obj* __p;
+	      // Try to make do with what we have.  That can't hurt.  We
+	      // do not try smaller requests, since that tends to result
+	      // in disaster on multi-process machines.
+	      __i = __size;
+	      for (; __i <= (size_t) _MAX_BYTES; __i += (size_t) _ALIGN) 
+		{
+		  __my_free_list = _S_free_list + _S_freelist_index(__i);
+		  __p = *__my_free_list;
+		  if (0 != __p) 
+		    {
+		      *__my_free_list = __p -> _M_free_list_link;
+		      _S_start_free = (char*)__p;
+		      _S_end_free = _S_start_free + __i;
+		      return(_S_chunk_alloc(__size, __nobjs));
+		      // Any leftover piece will eventually make it to the
+		      // right free list.
+		    }
+		}
+	      _S_end_free = 0;	// In case of exception.
+	      _S_start_free = (char*)__mem_interface::allocate(__bytes_to_get);
+	      // This should either throw an exception or remedy the situation.
+	      // Thus we assume it succeeded.
+	    }
+	  _S_heap_size += __bytes_to_get;
+	  _S_end_free = _S_start_free + __bytes_to_get;
+	  return(_S_chunk_alloc(__size, __nobjs));
+	}
+    }
+  
+  
+  // Returns an object of size __n, and optionally adds to "size
+  // __n"'s free list.  We assume that __n is properly aligned.  We
+  // hold the allocation lock.
+  template<bool __threads, int __inst>
+    void*
+    __default_alloc_template<__threads, __inst>::_S_refill(size_t __n)
+    {
+      int __nobjs = 20;
+      char* __chunk = _S_chunk_alloc(__n, __nobjs);
+      _Obj* volatile* __my_free_list;
+      _Obj* __result;
+      _Obj* __current_obj;
+      _Obj* __next_obj;
+      int __i;
+      
+      if (1 == __nobjs) return(__chunk);
+      __my_free_list = _S_free_list + _S_freelist_index(__n);
+      
+      /* Build free list in chunk */
       __result = (_Obj*)__chunk;
       *__my_free_list = __next_obj = (_Obj*)(__chunk + __n);
       for (__i = 1; ; __i++) {
         __current_obj = __next_obj;
         __next_obj = (_Obj*)((char*)__next_obj + __n);
         if (__nobjs - 1 == __i) {
-            __current_obj -> _M_free_list_link = 0;
-            break;
+	  __current_obj -> _M_free_list_link = 0;
+	  break;
         } else {
-            __current_obj -> _M_free_list_link = __next_obj;
+	  __current_obj -> _M_free_list_link = __next_obj;
         }
       }
-    return(__result);
-}
-
-
-template <bool threads, int inst>
-void*
-__default_alloc_template<threads, inst>::reallocate(void* __p,
-                                                    size_t __old_sz,
-                                                    size_t __new_sz)
-{
-    void* __result;
-    size_t __copy_sz;
-
-    if (__old_sz > (size_t) _MAX_BYTES && __new_sz > (size_t) _MAX_BYTES) {
-        return(realloc(__p, __new_sz));
+      return(__result);
     }
-    if (_S_round_up(__old_sz) == _S_round_up(__new_sz)) return(__p);
-    __result = allocate(__new_sz);
-    __copy_sz = __new_sz > __old_sz? __old_sz : __new_sz;
-    memcpy(__result, __p, __copy_sz);
-    deallocate(__p, __old_sz);
-    return(__result);
-}
 
-template <bool __threads, int __inst>
+
+  template<bool threads, int inst>
+    void*
+    __default_alloc_template<threads, inst>::reallocate(void* __p, 
+							size_t __old_sz,
+							size_t __new_sz)
+    {
+      void* __result;
+      size_t __copy_sz;
+      
+      if (__old_sz > (size_t) _MAX_BYTES && __new_sz > (size_t) _MAX_BYTES) {
+        return(realloc(__p, __new_sz));
+      }
+      if (_S_round_up(__old_sz) == _S_round_up(__new_sz)) return(__p);
+      __result = allocate(__new_sz);
+      __copy_sz = __new_sz > __old_sz? __old_sz : __new_sz;
+      memcpy(__result, __p, __copy_sz);
+      deallocate(__p, __old_sz);
+      return(__result);
+    }
+  
+  template<bool __threads, int __inst>
   _STL_mutex_lock
   __default_alloc_template<__threads, __inst>::_S_node_allocator_lock
-    __STL_MUTEX_INITIALIZER;
-
-template <bool __threads, int __inst>
-char* __default_alloc_template<__threads, __inst>::_S_start_free = 0;
-
-template <bool __threads, int __inst>
-char* __default_alloc_template<__threads, __inst>::_S_end_free = 0;
-
-template <bool __threads, int __inst>
-size_t __default_alloc_template<__threads, __inst>::_S_heap_size = 0;
-
-template <bool __threads, int __inst>
-typename __default_alloc_template<__threads, __inst>::_Obj* volatile
-__default_alloc_template<__threads, __inst> ::_S_free_list[
-    __default_alloc_template<__threads, __inst>::_NFREELISTS ];
-
-
-typedef __default_alloc_template<true, 0>    __alloc;
-typedef __default_alloc_template<false, 0>   __single_client_alloc;
+  __STL_MUTEX_INITIALIZER;
+  
+  template<bool __threads, int __inst>
+  char* __default_alloc_template<__threads, __inst>::_S_start_free = 0;
+  
+  template<bool __threads, int __inst>
+  char* __default_alloc_template<__threads, __inst>::_S_end_free = 0;
+  
+  template<bool __threads, int __inst>
+  size_t __default_alloc_template<__threads, __inst>::_S_heap_size = 0;
+  
+  template<bool __threads, int __inst>
+  typename __default_alloc_template<__threads, __inst>::_Obj* volatile
+  __default_alloc_template<__threads, __inst>::_S_free_list[_NFREELISTS];
+  
+  typedef __default_alloc_template<true, 0>    __alloc;
+  typedef __default_alloc_template<false, 0>   __single_client_alloc;
 
 
 #endif /* ! __USE_MALLOC */
@@ -674,12 +682,14 @@ inline bool operator!=(const allocator<_T1>&, const allocator<_T2>&)
 
 
 /**
+ *  @maint
  *  Allocator adaptor to turn an "SGI" style allocator (e.g., __alloc,
  *  __malloc_alloc_template) into a "standard" conforming allocator.  Note
  *  that this adaptor does *not* assume that all objects of the underlying
  *  alloc class are identical, nor does it assume that all of the underlying
  *  alloc's member functions are static member functions.  Note, also, that
  *  __allocator<_Tp, __alloc> is essentially the same thing as allocator<_Tp>.
+ *  @endmaint
  *  (See @link Allocators allocators info @endlink for more.)
 */
 template <class _Tp, class _Alloc>
@@ -822,9 +832,10 @@ inline bool operator!=(const __debug_alloc<_Alloc>&,
  *  The size_t parameters are "standard" style (see top of stl_alloc.h) in
  *  that they take counts, not sizes.
  *
- *  (See @link Allocators allocators info @endlink for more.)
  *  @endmaint
+ *  (See @link Allocators allocators info @endlink for more.)
 */
+//@{
 // The fully general version.
 template <class _Tp, class _Allocator>
 struct _Alloc_traits
@@ -844,6 +855,7 @@ struct _Alloc_traits<_Tp, allocator<_Tp1> >
   typedef __simple_alloc<_Tp, __alloc> _Alloc_type;
   typedef allocator<_Tp> allocator_type;
 };
+//@}
 
 //@{
 /// Versions for the predefined "SGI" style allocators.
