@@ -1,6 +1,6 @@
 // Iterators -*- C++ -*-
 
-// Copyright (C) 2001 Free Software Foundation, Inc.
+// Copyright (C) 2001, 2002 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
@@ -56,6 +56,10 @@
 /** @file stl_iterator.h
  *  This is an internal header file, included by other library headers.
  *  You should not attempt to use it directly.
+ *
+ *  This file implements reverse_iterator, back_insert_iterator,
+ *  front_insert_iterator, insert_iterator, __normal_iterator, and their
+ *  supporting functions and overloaded operators.
  */
 
 #ifndef __GLIBCPP_INTERNAL_ITERATOR_H
@@ -64,13 +68,31 @@
 namespace std
 {
   // 24.4.1 Reverse iterators
+  /**
+   *  "Bidirectional and random access iterators have corresponding reverse
+   *  %iterator adaptors that iterate through the data structure in the
+   *  opposite direction.  They have the same signatures as the corresponding
+   *  iterators.  The fundamental relation between a reverse %iterator and its
+   *  corresponding %iterator @c i is established by the identity:
+   *  @code
+   *      &*(reverse_iterator(i)) == &*(i - 1)
+   *  @endcode
+   *
+   *  This mapping is dictated by the fact that while there is always a
+   *  pointer past the end of an array, there might not be a valid pointer
+   *  before the beginning of an array." [24.4.1]/1,2
+   *
+   *  Reverse iterators can be tricky and surprising at first.  Their
+   *  semantics make sense, however, and the trickiness is a side effect of
+   *  the requirement that the iterators must be safe.
+  */
   template<typename _Iterator>
     class reverse_iterator 
-      : public iterator<typename iterator_traits<_Iterator>::iterator_category,
-			typename iterator_traits<_Iterator>::value_type,
-		        typename iterator_traits<_Iterator>::difference_type,
-		        typename iterator_traits<_Iterator>::pointer,
-                        typename iterator_traits<_Iterator>::reference>
+    : public iterator<typename iterator_traits<_Iterator>::iterator_category,
+		      typename iterator_traits<_Iterator>::value_type,
+		      typename iterator_traits<_Iterator>::difference_type,
+		      typename iterator_traits<_Iterator>::pointer,
+                      typename iterator_traits<_Iterator>::reference>
     {
     protected:
       _Iterator current;
@@ -83,21 +105,42 @@ namespace std
       typedef typename iterator_traits<_Iterator>::pointer     pointer;
 
     public:
-      reverse_iterator() {}
+      /**
+       *  The default constructor gives an undefined state to this %iterator.
+      */
+      reverse_iterator() { }
 
+      /**
+       *  This %iterator will move in the opposite direction that @p x does.
+      */
       explicit 
-      reverse_iterator(iterator_type __x) : current(__x) {}
+      reverse_iterator(iterator_type __x) : current(__x) { }
 
+      /**
+       *  The copy constructor is normal.
+      */
       reverse_iterator(const reverse_iterator& __x) 
-	: current(__x.current) { }
+      : current(__x.current) { }
 
+      /**
+       *  A reverse_iterator across other types can be copied in the normal
+       *  fashion.
+      */
       template<typename _Iter>
         reverse_iterator(const reverse_iterator<_Iter>& __x)
-	: current(__x.base()) {}
+	: current(__x.base()) { }
     
+      /**
+       *  @return  @c current, the %iterator used for underlying work.
+      */
       iterator_type 
       base() const { return current; }
 
+      /**
+       *  @return  TODO
+       *
+       *  @doctodo
+      */
       reference 
       operator*() const 
       {
@@ -105,9 +148,19 @@ namespace std
 	return *--__tmp;
       }
 
+      /**
+       *  @return  TODO
+       *
+       *  @doctodo
+      */
       pointer 
       operator->() const { return &(operator*()); }
 
+      /**
+       *  @return  TODO
+       *
+       *  @doctodo
+      */
       reverse_iterator& 
       operator++() 
       {
@@ -115,6 +168,11 @@ namespace std
 	return *this;
       }
 
+      /**
+       *  @return  TODO
+       *
+       *  @doctodo
+      */
       reverse_iterator 
       operator++(int) 
       {
@@ -123,6 +181,11 @@ namespace std
 	return __tmp;
       }
 
+      /**
+       *  @return  TODO
+       *
+       *  @doctodo
+      */
       reverse_iterator& 
       operator--() 
       {
@@ -130,6 +193,11 @@ namespace std
 	return *this;
       }
 
+      /**
+       *  @return  TODO
+       *
+       *  @doctodo
+      */
       reverse_iterator operator--(int) 
       {
 	reverse_iterator __tmp = *this;
@@ -137,10 +205,20 @@ namespace std
 	return __tmp;
       }
       
+      /**
+       *  @return  TODO
+       *
+       *  @doctodo
+      */
       reverse_iterator 
       operator+(difference_type __n) const 
       { return reverse_iterator(current - __n); }
 
+      /**
+       *  @return  TODO
+       *
+       *  @doctodo
+      */
       reverse_iterator& 
       operator+=(difference_type __n) 
       {
@@ -148,21 +226,46 @@ namespace std
 	return *this;
       }
 
+      /**
+       *  @return  TODO
+       *
+       *  @doctodo
+      */
       reverse_iterator 
       operator-(difference_type __n) const 
       { return reverse_iterator(current + __n); }
 
+      /**
+       *  @return  TODO
+       *
+       *  @doctodo
+      */
       reverse_iterator& 
       operator-=(difference_type __n) 
       {
-	_M_current += __n;
+	current += __n;
 	return *this;
       }
 
+      /**
+       *  @return  TODO
+       *
+       *  @doctodo
+      */
       reference 
       operator[](difference_type __n) const { return *(*this + __n); }  
     }; 
  
+  //@{
+  /**
+   *  @param  x  A %reverse_iterator.
+   *  @param  y  A %reverse_iterator.
+   *  @return  A simple bool.
+   *
+   *  Reverse iterators forward many operations to their underlying base()
+   *  iterators.  Others are implemented in terms of one another.
+   *
+  */
   template<typename _Iterator>
     inline bool 
     operator==(const reverse_iterator<_Iterator>& __x, 
@@ -210,21 +313,43 @@ namespace std
     operator+(typename reverse_iterator<_Iterator>::difference_type __n,
 	      const reverse_iterator<_Iterator>& __x) 
     { return reverse_iterator<_Iterator>(__x.base() - __n); }
+  //@}
 
   // 24.4.2.2.1 back_insert_iterator
+  /**
+   *  These are output iterators, constructed from a container-of-T.
+   *  Assigning a T to the iterator appends it to the container using
+   *  push_back.
+   *
+   *  Tip:  Using the back_inserter function to create these iterators can
+   *  save typing.
+  */
   template<typename _Container>
-  class back_insert_iterator 
+    class back_insert_iterator 
     : public iterator<output_iterator_tag, void, void, void, void>
     {
     protected:
       _Container* container;
 
     public:
+      /// A nested typedef for the type of whatever container you used.
       typedef _Container          container_type;
       
+      /// The only way to create this %iterator is with a container.
       explicit 
-      back_insert_iterator(_Container& __x) : container(&__x) {}
+      back_insert_iterator(_Container& __x) : container(&__x) { }
 
+      /**
+       *  @param  value  An instance of whatever type
+       *                 container_type::const_reference is; presumably a
+       *                 reference-to-const T for container<T>.
+       *  @return  This %iterator, for chained operations.
+       *
+       *  This kind of %iterator doesn't really have a "position" in the
+       *  container (you can think of the position as being permanently at
+       *  the end, if you like).  Assigning a value to the %iterator will
+       *  always append the value to the end of the container.
+      */
       back_insert_iterator&
       operator=(typename _Container::const_reference __value) 
       { 
@@ -232,33 +357,68 @@ namespace std
 	return *this;
       }
 
+      /// Simply returns *this.
       back_insert_iterator& 
       operator*() { return *this; }
 
+      /// Simply returns *this.  (This %iterator does not "move".)
       back_insert_iterator& 
       operator++() { return *this; }
 
+      /// Simply returns *this.  (This %iterator does not "move".)
       back_insert_iterator
       operator++(int) { return *this; }
     };
 
+  /**
+   *  @param  x  A container of arbitrary type.
+   *  @return  An instance of back_insert_iterator working on @p x.
+   *
+   *  This wrapper function helps in creating back_insert_iterator instances.
+   *  Typing the name of the %iterator requires knowing the precise full
+   *  type of the container, which can be tedious and impedes generic
+   *  programming.  Using this function lets you take advantage of automatic
+   *  template parameter deduction, making the compiler match the correct
+   *  types for you.
+  */
   template<typename _Container>
     inline back_insert_iterator<_Container> 
     back_inserter(_Container& __x) 
     { return back_insert_iterator<_Container>(__x); }
 
+  /**
+   *  These are output iterators, constructed from a container-of-T.
+   *  Assigning a T to the iterator prepends it to the container using
+   *  push_front.
+   *
+   *  Tip:  Using the front_inserter function to create these iterators can
+   *  save typing.
+  */
   template<typename _Container>
     class front_insert_iterator 
-      : public iterator<output_iterator_tag, void, void, void, void>
+    : public iterator<output_iterator_tag, void, void, void, void>
     {
     protected:
       _Container* container;
 
     public:
+      /// A nested typedef for the type of whatever container you used.
       typedef _Container          container_type;
 
-      explicit front_insert_iterator(_Container& __x) : container(&__x) {}
+      /// The only way to create this %iterator is with a container.
+      explicit front_insert_iterator(_Container& __x) : container(&__x) { }
 
+      /**
+       *  @param  value  An instance of whatever type
+       *                 container_type::const_reference is; presumably a
+       *                 reference-to-const T for container<T>.
+       *  @return  This %iterator, for chained operations.
+       *
+       *  This kind of %iterator doesn't really have a "position" in the
+       *  container (you can think of the position as being permanently at
+       *  the front, if you like).  Assigning a value to the %iterator will
+       *  always prepend the value to the front of the container.
+      */
       front_insert_iterator&
       operator=(typename _Container::const_reference __value) 
       { 
@@ -266,34 +426,89 @@ namespace std
 	return *this;
       }
 
+      /// Simply returns *this.
       front_insert_iterator& 
       operator*() { return *this; }
 
+      /// Simply returns *this.  (This %iterator does not "move".)
       front_insert_iterator& 
       operator++() { return *this; }
 
+      /// Simply returns *this.  (This %iterator does not "move".)
       front_insert_iterator 
       operator++(int) { return *this; }
     };
 
+  /**
+   *  @param  x  A container of arbitrary type.
+   *  @return  An instance of front_insert_iterator working on @p x.
+   *
+   *  This wrapper function helps in creating front_insert_iterator instances.
+   *  Typing the name of the %iterator requires knowing the precise full
+   *  type of the container, which can be tedious and impedes generic
+   *  programming.  Using this function lets you take advantage of automatic
+   *  template parameter deduction, making the compiler match the correct
+   *  types for you.
+  */
   template<typename _Container>
-  inline front_insert_iterator<_Container> front_inserter(_Container& __x) 
-  { return front_insert_iterator<_Container>(__x); }
+    inline front_insert_iterator<_Container> 
+    front_inserter(_Container& __x) 
+    { return front_insert_iterator<_Container>(__x); }
 
+  /**
+   *  These are output iterators, constructed from a container-of-T.
+   *  Assigning a T to the iterator inserts it in the container at the
+   *  %iterator's position, rather than overwriting the value at that
+   *  position.
+   *
+   *  (Sequences will actually insert a @e copy of the value before the
+   *  %iterator's position.)
+   *
+   *  Tip:  Using the inserter function to create these iterators can
+   *  save typing.
+  */
   template<typename _Container>
     class insert_iterator 
-      : public iterator<output_iterator_tag, void, void, void, void>
+    : public iterator<output_iterator_tag, void, void, void, void>
     {
     protected:
       _Container* container;
       typename _Container::iterator iter;
 
     public:
+      /// A nested typedef for the type of whatever container you used.
       typedef _Container          container_type;
       
+      /**
+       *  The only way to create this %iterator is with a container and an
+       *  initial position (a normal %iterator into the container).
+      */
       insert_iterator(_Container& __x, typename _Container::iterator __i) 
-	: container(&__x), iter(__i) {}
+      : container(&__x), iter(__i) {}
    
+      /**
+       *  @param  value  An instance of whatever type
+       *                 container_type::const_reference is; presumably a
+       *                 reference-to-const T for container<T>.
+       *  @return  This %iterator, for chained operations.
+       *
+       *  This kind of %iterator maintains its own position in the
+       *  container.  Assigning a value to the %iterator will insert the
+       *  value into the container at the place before the %iterator.
+       *
+       *  The position is maintained such that subsequent assignments will
+       *  insert values immediately after one another.  For example,
+       *  @code
+       *     // vector v contains A and Z
+       *
+       *     insert_iterator i (v, ++v.begin());
+       *     i = 1;
+       *     i = 2;
+       *     i = 3;
+       *
+       *     // vector v contains A, 1, 2, 3, and Z
+       *  @endcode
+      */
       insert_iterator&
       operator=(const typename _Container::const_reference __value) 
       { 
@@ -302,22 +517,36 @@ namespace std
 	return *this;
       }
 
+      /// Simply returns *this.
       insert_iterator& 
       operator*() { return *this; }
 
+      /// Simply returns *this.  (This %iterator does not "move".)
       insert_iterator& 
       operator++() { return *this; }
 
+      /// Simply returns *this.  (This %iterator does not "move".)
       insert_iterator& 
       operator++(int) { return *this; }
     };
   
+  /**
+   *  @param  x  A container of arbitrary type.
+   *  @return  An instance of insert_iterator working on @p x.
+   *
+   *  This wrapper function helps in creating insert_iterator instances.
+   *  Typing the name of the %iterator requires knowing the precise full
+   *  type of the container, which can be tedious and impedes generic
+   *  programming.  Using this function lets you take advantage of automatic
+   *  template parameter deduction, making the compiler match the correct
+   *  types for you.
+  */
   template<typename _Container, typename _Iterator>
-    inline 
-    insert_iterator<_Container> inserter(_Container& __x, _Iterator __i)
+    inline insert_iterator<_Container> 
+    inserter(_Container& __x, _Iterator __i)
     {
-      typedef typename _Container::iterator __iter;
-      return insert_iterator<_Container>(__x, __iter(__i));
+      return insert_iterator<_Container>(__x, 
+					 typename _Container::iterator(__i));
     }
   
   // This iterator adapter is 'normal' in the sense that it does not
